/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.jcr.Namespace;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import java.io.IOException;

import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;
import javax.jcr.Workspace;
import javax.servlet.jsp.JspException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Iterates over the namespaces that are available.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class ForEachNamespaceTag extends JcrSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(ForEachNamespaceTag.class);

    private NamespaceRegistry namespaceRegistry;
    private String var;

    @Override
    public void doTag() throws JspException, IOException {
        try {
            NamespaceRegistry determinedNamespaceRegistry = determineNamespaceRegistry();
            if (determinedNamespaceRegistry != null) {
                String[] uris = determinedNamespaceRegistry.getURIs();
                if (uris != null) {
                    String determinedVar = determineVar();
                    for (String uri : uris) {
                        String prefix = determinedNamespaceRegistry.getPrefix(uri);
                        setPageAttribute(determinedVar, new Namespace(prefix, uri));
                        getJspBody().invoke(null);
                    }
                }
            }
        } catch (RepositoryException e) {
            throw new JspException(e);
        }
    }

    /**
     * Accessor for the <code>namespaceRegistry</code> property.
     * 
     * @return the current value of the <code>namespaceRegistry</code> property.
     */
    public NamespaceRegistry getNamespaceRegistry() {
        return namespaceRegistry;
    }

    /**
     * Accessor for the <code>var</code> property.
     * 
     * @return the current value of the <code>var</code> property.
     */
    public String getVar() {
        return var;
    }

    /**
     * The {@link NamespaceRegistry} to use to find the namespaces. If a value is not supplied, the tag will attempt to
     * get the registry from the current JCR workspace.
     * 
     * @param namespaceRegistry
     *            the new value for the <code>namespaceRegistry</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setNamespaceRegistry(NamespaceRegistry namespaceRegistry) {
        this.namespaceRegistry = namespaceRegistry;
    }

    /**
     * The name of the page attribute that will hold the {@link Namespace} object. If no value is set, the tag will use
     * {@link AttributeNames#CURRENT_NAMESPACE}.
     * 
     * @param var
     *            the new value for the <code>var</code> property.
     */
    @JspTagAttribute(required = true)
    public void setVar(String var) {
        this.var = var;
    }

    private NamespaceRegistry determineNamespaceRegistry() throws RepositoryException {
        NamespaceRegistry result = null;
        if (namespaceRegistry != null) {
            result = namespaceRegistry;
        } else {
            Workspace workspace = getWorkspace();
            if (workspace != null) {
                result = workspace.getNamespaceRegistry();
            }
        }
        return result;
    }

    private String determineVar() {
        String result = var;
        if (result == null) {
            result = AttributeNames.CURRENT_NAMESPACE;
        }
        return result;
    }

}
