/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jcr.NodeUtils;
import com.squeakysand.jcr.SimpleNode;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Iterates over a collection of {@link Node} objects.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class ForEachNodeTag extends SimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(ForEachNodeTag.class);

    private NodeIterator nodeIterator;
    private String var;
    private Node previousCurrentNode;

    @Override
    public void doTag() throws JspException, IOException {
        savePreviousCurrentNode();
        List<SimpleNode> nodes = determineNodes();
        String attributeName = determineAttributeName();
        for (SimpleNode simpleNode : nodes) {
            getJspContext().setAttribute(attributeName, simpleNode);
            getJspBody().invoke(null);
        }
        restorePreviousCurrentNode();
    }

    public NodeIterator getNodeIterator() {
        return nodeIterator;
    }

    public String getVar() {
        return var;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setNodeIterator(NodeIterator nodeIterator) {
        this.nodeIterator = nodeIterator;
    }

    @JspTagAttribute
    public void setVar(String var) {
        this.var = var;
    }

    private String determineAttributeName() {
        String result = null;
        if (StringUtils.isNullOrEmpty(var)) {
            result = AttributeNames.CURRENT_NODE;
        } else {
            result = var;
        }
        return result;
    }

    private List<SimpleNode> determineNodes() {
        List<SimpleNode> result = null;
        if ((nodeIterator != null) && nodeIterator.hasNext()) {
            result = NodeUtils.asList(nodeIterator);
        }
        if (result == null) {
            result = Collections.emptyList();
        }
        return result;
    }

    private void restorePreviousCurrentNode() {
        if (previousCurrentNode != null) {
            getJspContext().setAttribute(AttributeNames.CURRENT_NODE, previousCurrentNode);
        } else {
            getJspContext().removeAttribute(AttributeNames.CURRENT_NODE);
        }
        previousCurrentNode = null;
    }

    private void savePreviousCurrentNode() throws JspException {
        try {
            previousCurrentNode = (Node) getJspContext().findAttribute(AttributeNames.CURRENT_NODE);
        } catch (ClassCastException e) {
            LOG.error(e.getMessage(), e);
        }
    }

}
