/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import javax.jcr.Session;
import javax.jcr.Workspace;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Parent class for tags that need access to the current JCR session.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
public abstract class JcrSimpleTagSupport extends EnhancedSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(JcrSimpleTagSupport.class);

    private Session session;
    private Workspace workspace;

    /**
     * Accessor for the <code>jcrSession</code> property.
     * 
     * @return the current value of the <code>jcrSession</code> property.
     */
    public Session getJcrSession() {
        return determineSession();
    }

    /**
     * Accessor for the <code>workspace</code> property.
     * 
     * @return the current value of the <code>workspace</code> property.
     */
    public Workspace getWorkspace() {
        return determineWorkspace();
    }

    /**
     * The JCR session to use. If this value is not set, the tag will attempt to find a session in the page context
     * under the name {@link AttributeNames#CURRENT_SESSION}.
     * 
     * @param jcrSession
     *            the new value for the <code>jcrSession</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setJcrSession(Session jcrSession) {
        session = jcrSession;
    }

    /**
     * Mutator for the <code>workspace</code> property.
     * 
     * @param workspace
     *            the new value for the <code>workspace</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setWorkspace(Workspace workspace) {
        this.workspace = workspace;
    }

    private Session determineSession() {
        Session result = null;
        if (session != null) {
            result = session;
        } else {
            try {
                result = findAttribute(AttributeNames.CURRENT_SESSION, Session.class);
            } catch (ClassCastException e) {
                LOG.error(e.getMessage(), e);
            }
        }
        return result;
    }

    private Workspace determineWorkspace() {
        Workspace result = null;
        if (workspace != null) {
            result = workspace;
        } else {
            Session session = getJcrSession();
            if (session != null) {
                result = session.getWorkspace();
            }
        }
        return result;
    }

}
